# -*- coding: utf-8 -*-
#
# django-codenerix-vending
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path as url

from .views import VendingInit
from .views import VendingCreateBudgerForRegisterUser, VendingCreateBudgerForEmployeesUser, VendingCreateBudgerForDefaultUser

"""
from .views import VendingGetPOS, VendingLoadOperators
from .views import VendingStart, Vending, VendingProducts, VendingAddProduct, VendingBasketToOrder, VendingLoadProductSlot, VendingDeleteProduct
from .views import VendingDeleteComanda, VendingCharge, VendingValidateTicket, VendingUpdateProduct, VendingPrintTicket
from .views import VendingPayment, VendingLogin, VendingLogout, VendingSubcategory, VendingPack, VendingPackOption, VendingGetProductPackOption
from .views import VendingHistory, VendingHistoryTicket, VendingHistoryTicketLoad, VendingHistoryTicketPrint
from .views import VendingOpenCashRegister, VendingCashDiary, VendingCashDiaryMovement
from .views import VendingGetFeatureSpecial
"""

urlpatterns = [

    url(r'^vending$', VendingInit.as_view(), name='CDNX_vending_init'),
    url(r'^sale_registered_users$', VendingCreateBudgerForRegisterUser.as_view(), name='CDNX_vending_sale_resgistered_user'),
    url(r'^sale_default_users$', VendingCreateBudgerForDefaultUser.as_view(), name='CDNX_vending_sale_default_user'),
    url(r'^sale_employees_users$', VendingCreateBudgerForEmployeesUser.as_view(), name='CDNX_vending_sale_employees_user'),

]
"""
urlpatterns = [
    url(r'^vendinggetpos$', VendingGetPOS.as_view(), name='CDNX_vending_get_pos'),

    url(r'^vending-login$', VendingLogin.as_view(), name='CDNX_vending_login'),
    url(r'^vending-logout$', VendingLogout.as_view(), name='CDNX_vending_logout'),
    url(r'^vending_load_operators$', VendingLoadOperators.as_view(), name='CDNX_vending_load_operators'),
    url(r'^vending-cashdiary$', VendingCashDiary.as_view(), name='CDNX_vending_cashdiary'),
    url(r'^vending-cashdiary-movement$', VendingCashDiaryMovement.as_view(), name='CDNX_vending_cashdiary_movement'),

    url(r'^vending-start$', VendingStart.as_view(), name='CDNX_vending_start'),
    url(r'^vending/(?P<pk>\w+)$', Vending.as_view(), name='CDNX_vending'),
    url(r'^vending-charge/(?P<pk>\w+)$', VendingCharge.as_view(), name='CDNX_vending_charge'),
    url(r'^vending-history$', VendingHistory.as_view(), name='CDNX_vending_history'),

    url(r'^vendingproducts$', VendingProducts.as_view(), name='CDNX_vending_products'),
    url(r'^vendingsubcategory$', VendingSubcategory.as_view(), name='CDNX_vending_subcategory'),
    url(r'^vendingpack$', VendingPack.as_view(), name='CDNX_vending_pack'),
    url(r'^vendingpackoption$', VendingPackOption.as_view(), name='CDNX_vending_packoption'),
    url(r'^vendinggetpackoption$', VendingGetProductPackOption.as_view(), name='CDNX_vending_get_packoption'),
    url(r'^vendinggetfeaturespecial$', VendingGetFeatureSpecial.as_view(), name='CDNX_vending_get_feature_special'),


    url(r'^vendingaddproduct$', VendingAddProduct.as_view(), name='CDNX_vending_products_add'),
    url(r'^vendingbaskettoorder$', VendingBasketToOrder.as_view(), name='CDNX_vending_basket_to_order'),
    url(r'^vendingloadproductslot$', VendingLoadProductSlot.as_view(), name='CDNX_vending_load_product_slot'),
    url(r'^vendingdeleteproduct/(?P<pk>\w+)$', VendingDeleteProduct.as_view(), name='CDNX_vending_products_delete'),
    url(r'^vendingupdateproduct/(?P<pk>\w+)$', VendingUpdateProduct.as_view(), name='CDNX_vending_products_update'),
    url(r'^vendingdeletecomanda/(?P<pk>\w+)$', VendingDeleteComanda.as_view(), name='CDNX_vending_products_comanda'),
    url(r'^vendingvalidateticket/(?P<pk>\w+)$', VendingValidateTicket.as_view(), name='CDNX_vending_validate_ticket'),
    url(r'^vendingprintticket/(?P<pk>\w+)$', VendingPrintTicket.as_view(), name='CDNX_vending_print_ticket'),
    url(r'^vendingopencashregister/(?P<pk>\w+)$', VendingOpenCashRegister.as_view(), name='CDNX_vending_open_cash_register'),

    url(r'^vendingpayment$', VendingPayment.as_view(), name='CDNX_vending_payment'),

    url(r'^vendinghistoryticket$', VendingHistoryTicket.as_view(), name='CDNX_vending_history_ticket'),
    url(r'^vendinghistoryticketload$', VendingHistoryTicketLoad.as_view(), name='CDNX_vending_load_ticket_history'),
    url(r'^vendinghistoryticketprint$', VendingHistoryTicketPrint.as_view(), name='CDNX_vending_print_ticket_history'),
    
]
"""



# -*- coding: utf-8 -*-
#
# django-codenerix-vending
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import uuid
import hashlib
import random
import string
# from channels import Channel, Group

from django.db import models
from django.utils.encoding import smart_str
from django.utils.translation import gettext_lazy as _
from django.db.models import Q
from django.conf import settings

from jsonfield import JSONField

from codenerix.models import CodenerixModel



# -*- coding: utf-8 -*-
#
# django-codenerix-vending
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django import forms
from django.utils.translation import gettext_lazy as _

from codenerix.forms import GenModelForm
from codenerix_extensions.helpers import get_external_model
from codenerix.widgets import MultiStaticSelect

from codenerix_invoicing.models_sales import SalesBasket, Customer


class BudgetForRegiterUserForm(GenModelForm):
    class Meta:
        model = SalesBasket
        fields = ['customer', ]
        # autofill = {
        #     'address_delivery': ['select', 3, Customer.foreignkey_external_delivery()['related'], 'customer'],
        # }

    def __groups__(self):
        return [
            (
                _('Customer'), 12,
                ['customer', 12],
            )
        ]


class BudgetForEmployeesUserForm(GenModelForm):
    class Meta:
        model = SalesBasket
        fields = ['customer', ]
        # autofill = {
        #     'address_delivery': ['select', 3, Customer.foreignkey_external_delivery()['related'], 'customer'],
        # }

    def __groups__(self):
        return [
            (
                _('Customer'), 12,
                ['customer', 12],
            )
        ]



# -*- coding: utf-8 -*-
#
# django-codenerix-vending
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import datetime

from django.db.models import F, Q
from django.shortcuts import render
from django.utils.translation import gettext as _
from django.conf import settings

from codenerix.middleware import get_current_user
from codenerix_extensions.helpers import get_language_database
from codenerix_invoicing.models_cash import CashDiary
from codenerix_invoicing.models_sales import (
    SalesBasket,
    ROLE_BASKET_SHOPPINGCART,
    SalesLines,
)

# SalesLineBasket, SalesLineTicket,
from codenerix_invoicing.models_sales import SalesTicket, ReasonModification
from codenerix_pos.models import (
    POSSlot,
    POSHardware,
    KIND_POSHARDWARE_TICKET,
    POS,
    POSOperator,
)
from codenerix_products.models import Category, ProductFinal, TypeTax


class CodenerixVending(object):
    def __init__(self, request):
        self.request = request

    def printer_ticket(self, PDV, request, ticket_obj, comanda=None):
        context = {}
        printer = POSHardware.objects.filter(
            kind=KIND_POSHARDWARE_TICKET, poss=PDV
        ).first()
        if printer:
            config = printer.get_config()
            column = config.get("column", 48)
            cut = config.get("cut", False)

            num_impressions = ticket_obj.print_counter(get_current_user())
            context_hardware = self.create_ticket_to_print(
                request, ticket_obj, num_impressions, comanda, column, cut
            )
            printer.send(context_hardware)
            context = context_hardware
            context = {"msg": "OK"}
        else:
            context = {"error": _("Printer do not found")}
        return context

    def create_ticket_to_print(
        self,
        request,
        ticket_obj,
        num_impressions=None,
        comanda=None,
        column=48,
        cut=False,
    ):
        template_name = "codnerix_vending/ticket.html"
        context_hardware = {}

        lines = SalesLines.objects.filter(
            ticket=ticket_obj, quantity__gt=0, removed=False
        ).all()
        info = ticket_obj.calculate_price_doc_complete()
        prices = info.copy()
        taxes = {}
        for tax in info["taxes"]:
            name = TypeTax.objects.filter(tax=tax).first()
            if name is None:
                name = tax
            taxes[name] = info["taxes"][tax]
        prices["taxes"] = taxes
        LINEA = "-" * column

        just = column - 7 - 8 - 5  # quantity, amount, margin
        ctx = {
            "labels": {"cant": "Cant", "product": "Producto", "price": "Precio"},
            "cut": cut,
            "ticket": ticket_obj,
            "lines": lines,
            "prices": prices,
            "LINEA": LINEA,
            "comanda": comanda,
            "config": {
                "cant": {
                    "slice": ":{}".format(7),
                    "just": 7,
                },
                "product": {
                    "slice": ":{}".format(just),
                    "just": just,
                },
                "price": {
                    "slice": ":{}".format(8),
                    "just": 8,
                },
                "subtotal": {
                    "slice": ":{}".format(column - 9),
                    "just": column - 9,
                },
                "tax_porc": {
                    "slice": ":{}".format(15),
                    "just": 15,
                },
                "tax": {
                    "slice": ":{}".format(column - 22),
                    "just": column - 16,
                },
                "total": {
                    "slice": ":{}".format(column - 6),
                    "just": column - 6,
                },
            },
        }

        rendering = render(request, template_name, ctx)
        context_hardware["template"] = rendering._container[0].decode("utf-8")
        if ticket_obj.lock is False:
            filename = "logo_ticket_cif.png"
            datas = None
        elif num_impressions > 1:
            filename = "logo_ticket_cif_reprinter.png"
            datas = "info_fiscal.png"
        elif num_impressions == 1:
            filename = "logo_ticket_cif.png"
            datas = "info_fiscal.png"

        File = open("{}img/{}".format(settings.STATIC_ROOT, filename), "rb")
        logo = base64.b64encode(File.read()).decode()
        context_hardware["ctx"] = {
            "logo": ["image", logo],
        }
        File.close()
        if datas:
            File = open("{}img/{}".format(settings.STATIC_ROOT, datas), "rb")
            dataf = base64.b64encode(File.read()).decode()
            File.close()
            context_hardware["dataf"] = ["image", dataf]
        else:
            dataf = None

        """
        if not comanda:
            context_hardware['ctx']['barcode'] = ['barcode', {
                'code': str(ticket_obj.code).zfill(13),
                'bc': 'EAN13',
                'height': 64,
                'width': 3,
                'pos': u'BELOW',
                'font': u'A',
                'align_ct': True,
                'function_type': None
            }, ]
        """
        return context_hardware

    def get_POS(self):
        uuid = self.request.session.get("POS_client_UUID", None)
        pos = POS.objects.filter(uuid=uuid).first()
        if pos:
            return {"uuid": str(pos.uuid), "POS": pos}
        else:
            return {"uuid": "", "POS": None}

    def get_lang(self):
        return get_language_database()

    def get_categories(self):
        PDV = self.get_POS()["POS"]
        lang = self.get_lang()

        qs = (
            Category.objects.filter(
                **{
                    "show_menu": True,
                    "products__products_final__posproducts__group_product__poss": PDV,
                    "products__products_final__{}__public".format(lang): True,
                }
            )
            .values("pk", "{}__name".format(lang))
            .annotate(name=F("{}__name".format(lang)))
            .order_by("order")
            .distinct()
        )

        return qs

    def get_shoppingcarts(self, paid=False, slot=None, without_order=False):
        PDV = self.get_POS()["POS"]

        qs = SalesBasket.objects.filter(
            pos_slot__zone__poss=PDV, role=ROLE_BASKET_SHOPPINGCART, removed=False
        )
        if not paid:
            qs = qs.filter(
                Q(order_sales__isnull=True)
                | Q(
                    order_sales__payment__isnull=True,
                    order_sales__cash_movements__isnull=True,
                    order_sales__removed=False,
                )
            )
        if slot:
            qs = qs.filter(
                pos_slot__pk=slot,
            )
        if without_order:
            qs = qs.filter(
                order_sales__isnull=True,
            )
        return qs

    def get_last_shoppingcarts(self, slot_pk):
        qs = (
            self.get_shoppingcarts(paid=False)
            .filter(
                pos_slot__pk=slot_pk,
            )
            .last()
        )
        return qs

    def get_lines_shoppingcart(self, slot_pk):
        qs = (
            SalesLines.objects.filter(
                basket__pos_slot__pk=slot_pk,
                basket__order_sales__payment__isnull=True,
                basket__order_sales__cash_movements__isnull=True,
                basket__role=ROLE_BASKET_SHOPPINGCART,
                quantity__gt=0,
                removed=False,
                basket__removed=False,
            )
            .annotate(xorder=F("basket__order_sales"))
            .order_by("pk")
        )
        return qs

    def get_products_pack(self):
        lang = self.get_lang()
        PDV = self.get_POS()["POS"]

        qs = (
            ProductFinal.objects.filter(
                Q(
                    **{
                        "{}__public".format(lang): True,
                        "productfinals_option__isnull": False,
                        "posproducts__group_product__poss": PDV,
                    }
                ),
            )
            .distinct()
            .annotate(name=F("{}__name".format(lang)))
        )
        return qs

    def get_slots(self):
        PDV = self.get_POS()["POS"]
        qs = POSSlot.objects.filter(zone__poss=PDV)
        return qs

    def get_hardware(self, kind):
        PDV = self.get_POS()["POS"]
        qs = POSHardware.objects.filter(pos=PDV, kind=kind, enable=True).first()
        return qs

    def get_reasons_modification(self):
        qs = ReasonModification.objects.filter(enable=True).order_by("code")
        return qs

    def get_operators(self):
        context = {}
        PDV = self.get_POS()["POS"]
        if PDV:
            opts = POSOperator.objects.filter(pos=PDV, enable=True)
            operators = []
            for operator in opts:
                operators.append(
                    {
                        "pk": operator.pk,
                        "name": operator.external.user.__str__(),
                        "user": operator.external.user.username,
                    }
                )
            context["operators"] = operators
        else:
            context["error"] = _(
                "POS not registered or can not be identified, plese make sure that POSClient is running"
            )
        return context

    def get_ticket(self, ticket_pk):
        PDV = self.get_POS()["POS"]
        ticket = SalesTicket.objects.filter(
            pk=ticket_pk,
            removed=False,
            line_ticket_sales__removed=False,
            line_ticket_sales__line_order__removed=False,
            line_ticket_sales__line_order__order__removed=False,
            line_ticket_sales__line_order__order__budget__removed=False,
            line_ticket_sales__line_order__order__budget__pos_slot__zone__poss=PDV,
        ).first()
        return ticket

    def reprinter_ticket(self, ticket_pk, request):
        context = {}
        PDV = self.get_POS()["POS"]
        ticket = self.get_ticket(ticket_pk)

        if not ticket:
            context["error"] = _("Ticket invalid")
        else:
            context.update(self.printer_ticket(PDV, request, ticket))
        return context

    def get_info_ticket(self, ticket_pk):
        context = {}
        ticket = self.get_ticket(ticket_pk)

        if not ticket:
            context["error"] = _("Ticket invalid")
        else:
            context["code"] = ticket.code
            products = []
            for line in ticket.line_ticket_sales.filter(quantity__gt=0):
                tmp = {}
                tmp["quantity"] = line.quantity
                tmp["price"] = float(line.total)
                tmp["description"] = line.description.__str__()
                if line.line_order.line_order_option_sales.exists():
                    tmp["more"] = []
                    for opt in line.line_order.line_order_option_sales.all().order_by(
                        "product_option__order"
                    ):
                        tmp["more"].append(
                            {
                                "quantity": opt.quantity,
                                "product": opt.product_final.__str__(),
                            }
                        )
                else:
                    tmp["more"] = None
                products.append(tmp)

            context["products"] = products

            totales = {}
            prices = ticket.calculate_price_doc_complete()
            if "discounts" in totales:
                totales["discounts"] = [
                    {x: float(prices["discounts"][x])}
                    for x in prices["discounts"].keys()
                ][0]
            else:
                totales["discounts"] = {}
            if "equivalence_surcharges" in totales:
                totales["equivalence_surcharges"] = [
                    {x: float(prices["equivalence_surcharges"][x])}
                    for x in prices["equivalence_surcharges"].keys()
                ][0]
            else:
                totales["equivalence_surcharges"] = {}
            if "taxes" in totales:
                totales["taxes"] = [
                    {x: float(prices["taxes"][x])} for x in prices["taxes"].keys()
                ][0]
            else:
                totales["taxes"] = {}
            totales["subtotal"] = float(prices["subtotal"])
            totales["total"] = float(prices["total"])

            context["totales"] = totales
        return context

    def get_history_tickets(self, slot_pk, date_ini, date_end):
        context = {
            "error": None,
            "tickets": [],
        }
        PDV = self.get_POS()["POS"]

        if slot_pk:
            slot = POSSlot.objects.filter(pk=slot_pk, zone__poss=PDV).first()
        else:
            slot = None
        if slot:
            tickets = (
                SalesTicket.objects.filter(
                    line_ticket_sales__line_order__order__budget__pos_slot=slot,
                    removed=False,
                    line_ticket_sales__line_order__order__budget__removed=False,
                    line_ticket_sales__line_order__order__removed=False,
                    line_ticket_sales__line_order__removed=False,
                    line_ticket_sales__removed=False,
                    date__range=(
                        datetime.datetime.combine(date_ini, datetime.time.min),
                        datetime.datetime.combine(date_end, datetime.time.max),
                    ),
                )
                .order_by("-pk")
                .distinct()
            )
            for ticket in tickets:
                context["tickets"].append(
                    {
                        "pk": ticket.pk,
                        "code": ticket.code,
                        "total": float(ticket.total),
                    }
                )

        else:
            context["error"] = _("Slot invalid")

        return context

    def cashdiary_isopen(self):
        PDV = self.get_POS()["POS"]
        opened = CashDiary.objects.filter(pos=PDV, closed_user__isnull=True).first()
        return bool(opened)



# -*- coding: utf-8 -*-
#
# django-codenerix-vending
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import datetime

from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.urls import reverse
from django.views.generic import View
from django.forms.utils import ErrorList

from codenerix.views import GenCreateModal, GenCreate

from codenerix_invoicing.models import BillingSeries
from codenerix_invoicing.models_cash import CashDiary
from codenerix_invoicing.models_sales import SalesBasket, Customer
from codenerix_pos.models import POS
from .forms import BudgetForRegiterUserForm, BudgetForEmployeesUserForm


class VendingInit(View):
    template_name = "codenerix_vending/init.html"

    @method_decorator(login_required)
    def get(self, request, *args, **kwargs):
        context = {}

        uuid = self.request.session.get("POS_client_UUID", None)
        opened = CashDiary.objects.filter(
            pos__uuid=uuid, closed_user__isnull=True
        ).first()

        if opened is None:
            context["error"] = _("No CashDiary is open!")
        context["ws_entry_point"] = "codenerix_vending"
        context["url_budget"] = reverse(
            "CDNX_invoicing_salesbaskets_shoppingcart_vending_list",
            kwargs={"bpk": "CNDX_VENDING_LINES"},
        )

        return render(request, self.template_name, context)


class VendingCreateBudgerForRegisterUser(GenCreateModal, GenCreate):
    model = SalesBasket
    form_class = BudgetForRegiterUserForm

    def form_valid(self, form):
        bs = BillingSeries.objects.filter(default=True).first()
        if bs is None:
            errors = form._errors.setdefault("customer", ErrorList())
            errors.append(_("Billing Series by default not exists"))
            return self.form_invalid(form)
        else:
            self.request.billing_series = bs
            form.instance.billing_series = bs

            self.request.name = "{}".format(datetime.datetime.now())
            form.instance.name = "{}".format(datetime.datetime.now())

            self.request.signed = True
            form.instance.signed = True

            uuid = self.request.session.get("POS_client_UUID", None)
            pos = POS.objects.filter(uuid=uuid).first()

            self.request.pos = pos
            form.instance.pos = pos

        return super(VendingCreateBudgerForRegisterUser, self).form_valid(form)


class VendingCreateBudgerForEmployeesUser(VendingCreateBudgerForRegisterUser):
    form_class = BudgetForEmployeesUserForm


class VendingCreateBudgerForDefaultUser(View):
    def post(self, request, *args, **kwargs):
        answer = {}
        # falta el pos
        bs = BillingSeries.objects.filter(default=True).first()
        customer_default = Customer.objects.filter(default_customer=True).first()
        if bs is None:
            answer["error"] = _("Billing Series by default not exists")
        elif customer_default is None:
            answer["error"] = _("Customer by default not exists")
        else:
            budget = SalesBasket()
            budget.billing_series = bs
            budget.name = "{}".format(datetime.datetime.now())
            budget.signed = True
            budget.customer = customer_default
            uuid = self.request.session.get("POS_client_UUID", None)
            pos = POS.objects.filter(uuid=uuid).first()

            budget.pos = pos

            budget.save()
            answer["__pk__"] = budget.pk

        return JsonResponse(answer)
